@@ -56,29 +56,37 @@ GMLTimePeriod <- R6Class("GMLTimePeriod",
     endPosition = NULL,
     #+ duration [0..1]: character
     duration = NULL,
-    initialize = function(xml = NULL, beginPosition, endPosition){
+    initialize = function(xml = NULL, beginPosition = NULL, endPosition = NULL){
       super$initialize(xml = xml)
       if(is.null(xml)){
-        self$setBeginPosition(beginPosition)
-        self$setEndPosition(endPosition)
+        if(!is.null(beginPosition)) self$setBeginPosition(beginPosition)
+        if(!is.null(endPosition)) self$setEndPosition(endPosition)
       }
     },
     
     #setBeginPosition
     setBeginPosition = function(beginPosition){
-      if(!all(class(beginPosition)==c("POSIXct","POSIXt")) | is(beginPosition, "Date")){
-        stop("Value should be of class ('POSIXct','POSIXt') or 'Date'")
+      beginPos <- beginPosition
+      if(is(beginPos,"numeric")) beginPos <- as(beginPos, "character")
+      if(!(is(beginPos, "character") & nchar(beginPos) %in% c(4,7))){
+        if(!all(class(beginPos)==c("POSIXct","POSIXt")) | is(beginPos, "Date")){
+          stop("For a date, the value should be of class ('POSIXct','POSIXt') or 'Date'")
+        }
       }
-      self$beginPosition <- GMLElement$create("beginPosition", value = beginPosition)
+      self$beginPosition <- GMLElement$create("beginPosition", value = beginPos)
       if(!is.null(self$endPosition)) self$computeInterval()
     },
     
     #setEndPosition
     setEndPosition = function(endPosition){
-      if(!all(class(endPosition)==c("POSIXct","POSIXt")) | is(endPosition, "Date")){
-        stop("Value should be of class ('POSIXct','POSIXt') or 'Date'")
+      endPos <- endPosition
+      if(is(endPos,"numeric")) endPos <- as(endPos, "character")
+      if(!(is(endPos, "character") & nchar(endPos) %in% c(4,7))){
+        if(!all(class(endPos)==c("POSIXct","POSIXt")) | is(endPos, "Date")){
+          stop("Value should be of class ('POSIXct','POSIXt') or 'Date'")
+        }
       }
-      self$endPosition <- GMLElement$create("endPosition", value = endPosition)
+      self$endPosition <- GMLElement$create("endPosition", value = endPos)
       if(!is.null(self$beginPosition)) self$computeInterval()
     },
     
@@ -87,23 +95,42 @@ GMLTimePeriod <- R6Class("GMLTimePeriod",
       
       start <- self$beginPosition$value
       end <- self$endPosition$value
-      years.seq <- seq(start, end, by = "years")
-      years <- length(years.seq)-1
-      months.start <- years.seq[length(years.seq)]
-      months.seq <- seq(months.start, end, by = "months")
-      months <- length(months.seq)-1
-      days.start <- months.seq[length(months.seq)]
-      days.seq <- seq(days.start, end, by = "DSTdays")
-      days <- length(days.seq)-1
-      hours.start <- days.seq[length(days.seq)]
-      hours.seq <- seq(hours.start, end, by = "hours")
-      hours <- length(hours.seq)-1
-      mins.start <- hours.seq[length(hours.seq)]
-      mins.seq <- seq(mins.start, end, by = "mins")
-      mins <- length(mins.seq)-1
-      secs.start <- mins.seq[length(mins.seq)]
-      secs.seq <- seq(secs.start, end, by = "secs")
-      secs <- length(secs.seq)-1
+      if(nchar(start)==4 && nchar(end)==4){
+        years <- length(as.numeric(start):as.numeric(end))
+        months <- 0; days <- 0; hours <- 0; mins <- 0; secs <- 0;
+      }else if(nchar(start)==7 && nchar(end)==7){
+        isLeapYear = function(year){ return(((year %% 4 == 0) & (year %% 100 != 0)) | (year %% 400 == 0)) }
+        startYear <- as.numeric(substr(start, 1, 4))
+        startMonth <- as.numeric(substr(start,6,7))
+        startDate <- as.Date(paste(start,"01",sep="-"))
+        endYear <- as.numeric(substr(end, 1, 4))
+        endMonth <- as.numeric(substr(end,6,7))
+        endDate <- as.Date(paste(end,ifelse(isLeapYear(endYear)&&endMonth==2,"29","28"),sep="-"))
+        years.seq <- seq(startDate, endDate, by = "years")
+        years <- length(years.seq)-1
+        months.start <- years.seq[length(years.seq)]
+        months.seq <- seq(months.start, endDate, by = "months")
+        months <- length(months.seq)-1
+        days <- 0; hours <- 0; mins <- 0; secs <- 0;
+      }else{
+        years.seq <- seq(start, end, by = "years")
+        years <- length(years.seq)-1
+        months.start <- years.seq[length(years.seq)]
+        months.seq <- seq(months.start, end, by = "months")
+        months <- length(months.seq)-1
+        days.start <- months.seq[length(months.seq)]
+        days.seq <- seq(days.start, end, by = "DSTdays")
+        days <- length(days.seq)-1
+        hours.start <- days.seq[length(days.seq)]
+        hours.seq <- seq(hours.start, end, by = "hours")
+        hours <- length(hours.seq)-1
+        mins.start <- hours.seq[length(hours.seq)]
+        mins.seq <- seq(mins.start, end, by = "mins")
+        mins <- length(mins.seq)-1
+        secs.start <- mins.seq[length(mins.seq)]
+        secs.seq <- seq(secs.start, end, by = "secs")
+        secs <- length(secs.seq)-1
+      }
       isoduration <- GMLTimePeriod$computeISODuration(years, months, days, 
                                                       hours, mins, secs)           
       self$setId(isoduration, addNS = TRUE)
