@@ -22,4 +22,10 @@
   assign(".geometa.gml", new.env(), envir = asNamespace(pkgname))
   .geometa.gml$serialId <- 1L
   
+  #mappings
+  assign(".geometa.mappings", new.env(), envir = asNamespace(pkgname))
+  #mapping rules
+  mappings_file <- system.file("extdata/mappings", "geometa_mapping.csv", package = "geometa")
+  registerMappings(utils::read.csv(mappings_file, stringsAsFactors = FALSE, na.strings = ""))
+  
 } # nocov end
