@@ -10,6 +10,8 @@ require(XML)
 context("GMLAbstractSingleCRS")
 
 test_that("encoding",{
+  testthat::skip_on_cran()
+  testthat::skip_on_travis()
   
   #encoding
   gml <- GMLAbstractSingleCRS$new()
