@@ -9,7 +9,8 @@ require(testthat)
 context("ISOCarrierOfCharacteristics")
 
 test_that("encoding",{
-  
+  testthat::skip_on_cran()
+  testthat::skip_on_travis()
   #featuretype
   md <- ISOCarrierOfCharacteristics$new()
   md$setFeatureType(ISOFeatureType$new())
