@@ -9,7 +9,8 @@ require(testthat)
 context("ISODefinitionReference")
 
 test_that("encoding",{
-  
+  testthat::skip_on_cran()
+  testthat::skip_on_travis()
   #encoding
   md <- ISODefinitionReference$new()
   md$setSourceIdentifier("identifier")
