@@ -9,7 +9,9 @@ require(testthat)
 context("ISOFeatureCatalogueDescription")
 
 test_that("encoding",{
-  
+  testthat::skip_on_cran()
+  testthat::skip_on_travis()
+  #encoding
   md <- ISOFeatureCatalogueDescription$new()
   md$setComplianceCode(FALSE)
   md$addLanguage("eng")
