@@ -9,7 +9,8 @@ require(testthat)
 context("ISOGeographicDescription")
 
 test_that("encoding",{
-  
+  testthat::skip_on_cran()
+  testthat::skip_on_travis()
   #encoding
   md <- ISOGeographicDescription$new()
   md$setGeographicIdentifier(ISOMetaIdentifier$new(code = "identifier"))
