@@ -9,9 +9,10 @@ require(testthat)
 context("ISOGeoreferenceable")
 
 test_that("encoding",{
-  
+  testthat::skip_on_cran()
+  testthat::skip_on_travis()
+  #encodidng
   md <- ISOGeoreferenceable$new()
-  
   #inherited methods from ISOGridSpatialRepresentation
   md$setNumberOfDimensions(1)
   md$setTransformationParameterAvailability(FALSE)
