@@ -9,7 +9,8 @@ require(testthat)
 context("ISOUnlimitedInteger")
 
 test_that("encoding",{
-  
+  testthat::skip_on_cran()
+  testthat::skip_on_travis()
   #encoding
   md <- ISOUnlimitedInteger$new(value = 19L)
   expect_is(md, "ISOUnlimitedInteger")
