@@ -9,7 +9,8 @@ require(testthat)
 context("ISOVectorSpatialRepresentation")
 
 test_that("encoding",{
-  
+  testthat::skip_on_cran()
+  testthat::skip_on_travis()
   #encoding
   md <- ISOVectorSpatialRepresentation$new()
   md$setTopologyLevel("geometryOnly")
